@@ -124,6 +124,7 @@ Instr Istream::Read(Offset* offset) const {
     case Opcode::Nop:
     case Opcode::Return:
     case Opcode::Unreachable:
+    case Opcode::ThrowRef:
     case Opcode::RefNull:
       // 0 immediates, 0 operands.
       instr.kind = InstrKind::Imm_0_Op_0;
@@ -255,6 +256,10 @@ Instr Istream::Read(Offset* offset) const {
     case Opcode::I16X8ExtaddPairwiseI8X16U:
     case Opcode::I32X4ExtaddPairwiseI16X8S:
     case Opcode::I32X4ExtaddPairwiseI16X8U:
+    case Opcode::I32X4RelaxedTruncF32X4S:
+    case Opcode::I32X4RelaxedTruncF32X4U:
+    case Opcode::I32X4RelaxedTruncF64X2SZero:
+    case Opcode::I32X4RelaxedTruncF64X2UZero:
       // 0 immediates, 1 operand.
       instr.kind = InstrKind::Imm_0_Op_1;
       break;
@@ -468,12 +473,28 @@ Instr Istream::Read(Offset* offset) const {
     case Opcode::V128Or:
     case Opcode::V128Xor:
     case Opcode::I8X16Swizzle:
+    case Opcode::I8X16RelaxedSwizzle:
+    case Opcode::F32X4RelaxedMin:
+    case Opcode::F32X4RelaxedMax:
+    case Opcode::F64X2RelaxedMin:
+    case Opcode::F64X2RelaxedMax:
+    case Opcode::I16X8RelaxedQ15mulrS:
+    case Opcode::I16X8DotI8X16I7X16S:
       // 0 immediates, 2 operands
       instr.kind = InstrKind::Imm_0_Op_2;
       break;
 
     case Opcode::Select:
     case Opcode::SelectT:
+    case Opcode::F32X4RelaxedMadd:
+    case Opcode::F32X4RelaxedNmadd:
+    case Opcode::F64X2RelaxedMadd:
+    case Opcode::F64X2RelaxedNmadd:
+    case Opcode::I8X16RelaxedLaneSelect:
+    case Opcode::I16X8RelaxedLaneSelect:
+    case Opcode::I32X4RelaxedLaneSelect:
+    case Opcode::I64X2RelaxedLaneSelect:
+    case Opcode::I32X4DotI8X16I7X16AddS:
       // 0 immediates, 3 operands
       instr.kind = InstrKind::Imm_0_Op_3;
       break;
@@ -494,6 +515,9 @@ Instr Istream::Read(Offset* offset) const {
 
     case Opcode::GlobalGet:
     case Opcode::LocalGet:
+    case Opcode::InterpLocalGetRef:
+    case Opcode::InterpGlobalGetRef:
+    case Opcode::InterpMarkRef:
     case Opcode::MemorySize:
     case Opcode::TableSize:
     case Opcode::DataDrop:
@@ -773,6 +797,7 @@ Instr Istream::Read(Offset* offset) const {
     case Opcode::Invalid:
     case Opcode::Loop:
     case Opcode::Try:
+    case Opcode::TryTable:
     case Opcode::ReturnCall:
       // Not used.
       break;
