@@ -31,11 +31,11 @@ R"w2c_template(// A pointer for an object of size n.
 )w2c_template"
 R"w2c_template(#if WABT_BIG_ENDIAN
 )w2c_template"
-R"w2c_template(#define MEM_ADDR(mem, addr, n) &(mem)->data[(mem)->size - (addr) - (n)]
+R"w2c_template(#define MEM_ADDR(mem, addr, n) ((mem)->data_end - (addr) - (n))
 )w2c_template"
 R"w2c_template(#else
 )w2c_template"
-R"w2c_template(#define MEM_ADDR(mem, addr, n) &(mem)->data[addr]
+R"w2c_template(#define MEM_ADDR(mem, addr, n) &((mem)->data[addr])
 )w2c_template"
 R"w2c_template(#endif
 )w2c_template"
@@ -194,7 +194,7 @@ R"w2c_template(    if (UNLIKELY(add_overflow(offset, len, &res))) \
 )w2c_template"
 R"w2c_template(      TRAP(OOB);                                   \
 )w2c_template"
-R"w2c_template(    if (UNLIKELY(res > mem->size))                 \
+R"w2c_template(    if (UNLIKELY(res > (mem)->size))               \
 )w2c_template"
 R"w2c_template(      TRAP(OOB);                                   \
 )w2c_template"
@@ -285,7 +285,7 @@ R"w2c_template(#define FORCE_READ_FLOAT(var)
 R"w2c_template(#endif
 )w2c_template"
 R"w2c_template(
-static inline void load_data(void* dest, const void* src, size_t n) {
+static inline void load_data(u8* dest, const u8* src, size_t n) {
 )w2c_template"
 R"w2c_template(  if (!n) {
 )w2c_template"
@@ -293,22 +293,18 @@ R"w2c_template(    return;
 )w2c_template"
 R"w2c_template(  }
 )w2c_template"
-R"w2c_template(  wasm_rt_memcpy(dest, src, n);
-)w2c_template"
 R"w2c_template(#if WABT_BIG_ENDIAN
 )w2c_template"
-R"w2c_template(  u8* dest_chars = dest;
+R"w2c_template(  for (size_t i = 0; i < n; i++) {
 )w2c_template"
-R"w2c_template(  for (size_t i = 0; i < (n >> 1); i++) {
-)w2c_template"
-R"w2c_template(    u8 cursor = dest_chars[i];
-)w2c_template"
-R"w2c_template(    dest_chars[i] = dest_chars[n - i - 1];
-)w2c_template"
-R"w2c_template(    dest_chars[n - i - 1] = cursor;
+R"w2c_template(    dest[i] = src[n - i - 1];
 )w2c_template"
 R"w2c_template(  }
 )w2c_template"
+R"w2c_template(#else
+)w2c_template"
+R"w2c_template(  wasm_rt_memcpy(dest, src, n);
+)w2c_template"
 R"w2c_template(#endif
 )w2c_template"
 R"w2c_template(}
@@ -628,24 +624,22 @@ R"w2c_template(
 #endif
 )w2c_template"
 R"w2c_template(
-#define DIV_S(ut, min, x, y)                                      \
+#define DIV_S(ut, min, x, y)                                  \
 )w2c_template"
-R"w2c_template(  ((UNLIKELY((y) == 0))                                           \
+R"w2c_template(  ((UNLIKELY((y) == 0))                  ? TRAP(DIV_BY_ZERO)  \
 )w2c_template"
-R"w2c_template(       ? TRAP(DIV_BY_ZERO)                                        \
+R"w2c_template(   : (UNLIKELY((x) == min && (y) == -1)) ? TRAP(INT_OVERFLOW) \
 )w2c_template"
-R"w2c_template(       : (UNLIKELY((x) == min && (y) == -1)) ? TRAP(INT_OVERFLOW) \
-)w2c_template"
-R"w2c_template(                                             : (ut)((x) / (y)))
+R"w2c_template(                                         : (ut)((x) / (y)))
 )w2c_template"
 R"w2c_template(
-#define REM_S(ut, min, x, y) \
+#define REM_S(ut, min, x, y)                                 \
 )w2c_template"
-R"w2c_template(  ((UNLIKELY((y) == 0))      \
+R"w2c_template(  ((UNLIKELY((y) == 0))                  ? TRAP(DIV_BY_ZERO) \
 )w2c_template"
-R"w2c_template(       ? TRAP(DIV_BY_ZERO)   \
+R"w2c_template(   : (UNLIKELY((x) == min && (y) == -1)) ? 0                 \
 )w2c_template"
-R"w2c_template(       : (UNLIKELY((x) == min && (y) == -1)) ? 0 : (ut)((x) % (y)))
+R"w2c_template(                                         : (ut)((x) % (y)))
 )w2c_template"
 R"w2c_template(
 #define I32_DIV_S(x, y) DIV_S(u32, INT32_MIN, (s32)x, (s32)y)
@@ -685,45 +679,39 @@ R"w2c_template(#define I32_ROTR(x, y) ROTR(x, y, 31)
 R"w2c_template(#define I64_ROTR(x, y) ROTR(x, y, 63)
 )w2c_template"
 R"w2c_template(
-#define FMIN(x, y)                                                     \
+#define FMIN(x, y)                                           \
 )w2c_template"
-R"w2c_template(  ((UNLIKELY((x) != (x)))                                              \
+R"w2c_template(  ((UNLIKELY((x) != (x)))             ? NAN                  \
 )w2c_template"
-R"w2c_template(       ? NAN                                                           \
+R"w2c_template(   : (UNLIKELY((y) != (y)))           ? NAN                  \
 )w2c_template"
-R"w2c_template(       : (UNLIKELY((y) != (y)))                                        \
+R"w2c_template(   : (UNLIKELY((x) == 0 && (y) == 0)) ? (signbit(x) ? x : y) \
 )w2c_template"
-R"w2c_template(             ? NAN                                                     \
+R"w2c_template(   : (x < y)                          ? x                    \
 )w2c_template"
-R"w2c_template(             : (UNLIKELY((x) == 0 && (y) == 0)) ? (signbit(x) ? x : y) \
-)w2c_template"
-R"w2c_template(                                                : (x < y) ? x : y)
+R"w2c_template(                                      : y)
 )w2c_template"
 R"w2c_template(
-#define FMAX(x, y)                                                     \
+#define FMAX(x, y)                                           \
 )w2c_template"
-R"w2c_template(  ((UNLIKELY((x) != (x)))                                              \
+R"w2c_template(  ((UNLIKELY((x) != (x)))             ? NAN                  \
 )w2c_template"
-R"w2c_template(       ? NAN                                                           \
+R"w2c_template(   : (UNLIKELY((y) != (y)))           ? NAN                  \
 )w2c_template"
-R"w2c_template(       : (UNLIKELY((y) != (y)))                                        \
+R"w2c_template(   : (UNLIKELY((x) == 0 && (y) == 0)) ? (signbit(x) ? y : x) \
 )w2c_template"
-R"w2c_template(             ? NAN                                                     \
+R"w2c_template(   : (x > y)                          ? x                    \
 )w2c_template"
-R"w2c_template(             : (UNLIKELY((x) == 0 && (y) == 0)) ? (signbit(x) ? y : x) \
-)w2c_template"
-R"w2c_template(                                                : (x > y) ? x : y)
+R"w2c_template(                                      : y)
 )w2c_template"
 R"w2c_template(
-#define TRUNC_S(ut, st, ft, min, minop, max, x)                           \
+#define TRUNC_S(ut, st, ft, min, minop, max, x)                             \
 )w2c_template"
-R"w2c_template(  ((UNLIKELY((x) != (x)))                                                 \
+R"w2c_template(  ((UNLIKELY((x) != (x)))                        ? TRAP(INVALID_CONVERSION) \
 )w2c_template"
-R"w2c_template(       ? TRAP(INVALID_CONVERSION)                                         \
+R"w2c_template(   : (UNLIKELY(!((x)minop(min) && (x) < (max)))) ? TRAP(INT_OVERFLOW)       \
 )w2c_template"
-R"w2c_template(       : (UNLIKELY(!((x)minop(min) && (x) < (max)))) ? TRAP(INT_OVERFLOW) \
-)w2c_template"
-R"w2c_template(                                                     : (ut)(st)(x))
+R"w2c_template(                                                 : (ut)(st)(x))
 )w2c_template"
 R"w2c_template(
 #define I32_TRUNC_S_F32(x) \
@@ -743,15 +731,13 @@ R"w2c_template(#define I64_TRUNC_S_F64(x) \
 R"w2c_template(  TRUNC_S(u64, s64, f64, (f64)INT64_MIN, >=, (f64)INT64_MAX, x)
 )w2c_template"
 R"w2c_template(
-#define TRUNC_U(ut, ft, max, x)                                          \
+#define TRUNC_U(ut, ft, max, x)                                              \
 )w2c_template"
-R"w2c_template(  ((UNLIKELY((x) != (x)))                                                \
+R"w2c_template(  ((UNLIKELY((x) != (x)))                         ? TRAP(INVALID_CONVERSION) \
 )w2c_template"
-R"w2c_template(       ? TRAP(INVALID_CONVERSION)                                        \
+R"w2c_template(   : (UNLIKELY(!((x) > (ft) - 1 && (x) < (max)))) ? TRAP(INT_OVERFLOW)       \
 )w2c_template"
-R"w2c_template(       : (UNLIKELY(!((x) > (ft)-1 && (x) < (max)))) ? TRAP(INT_OVERFLOW) \
-)w2c_template"
-R"w2c_template(                                                    : (ut)(x))
+R"w2c_template(                                                  : (ut)(x))
 )w2c_template"
 R"w2c_template(
 #define I32_TRUNC_U_F32(x) TRUNC_U(u32, f32, 4294967296.f, x)
@@ -765,15 +751,13 @@ R"w2c_template(#define I64_TRUNC_U_F64(x) TRUNC_U(u64, f64, (f64)UINT64_MAX, x)
 R"w2c_template(
 #define TRUNC_SAT_S(ut, st, ft, min, smin, minop, max, smax, x) \
 )w2c_template"
-R"w2c_template(  ((UNLIKELY((x) != (x)))                                       \
+R"w2c_template(  ((UNLIKELY((x) != (x)))         ? 0                           \
 )w2c_template"
-R"w2c_template(       ? 0                                                      \
+R"w2c_template(   : (UNLIKELY(!((x)minop(min)))) ? smin                        \
 )w2c_template"
-R"w2c_template(       : (UNLIKELY(!((x)minop(min))))                           \
+R"w2c_template(   : (UNLIKELY(!((x) < (max))))   ? smax                        \
 )w2c_template"
-R"w2c_template(             ? smin                                             \
-)w2c_template"
-R"w2c_template(             : (UNLIKELY(!((x) < (max)))) ? smax : (ut)(st)(x))
+R"w2c_template(                                  : (ut)(st)(x))
 )w2c_template"
 R"w2c_template(
 #define I32_TRUNC_SAT_S_F32(x)                                            \
@@ -801,15 +785,15 @@ R"w2c_template(  TRUNC_SAT_S(u64, s64, f64, (f64)INT64_MIN, INT64_MIN, >=, (f64)
 R"w2c_template(              INT64_MAX, x)
 )w2c_template"
 R"w2c_template(
-#define TRUNC_SAT_U(ut, ft, max, smax, x)               \
+#define TRUNC_SAT_U(ut, ft, max, smax, x) \
 )w2c_template"
-R"w2c_template(  ((UNLIKELY((x) != (x))) ? 0                           \
+R"w2c_template(  ((UNLIKELY((x) != (x)))          ? 0    \
 )w2c_template"
-R"w2c_template(                          : (UNLIKELY(!((x) > (ft)-1))) \
+R"w2c_template(   : (UNLIKELY(!((x) > (ft) - 1))) ? 0    \
 )w2c_template"
-R"w2c_template(                                ? 0                     \
+R"w2c_template(   : (UNLIKELY(!((x) < (max))))    ? smax \
 )w2c_template"
-R"w2c_template(                                : (UNLIKELY(!((x) < (max)))) ? smax : (ut)(x))
+R"w2c_template(                                   : (ut)(x))
 )w2c_template"
 R"w2c_template(
 #define I32_TRUNC_SAT_U_F32(x) \
@@ -1126,9 +1110,12 @@ R"w2c_template(  LOAD_DATA((*dest), dest_addr, src + src_addr, n);
 R"w2c_template(}
 )w2c_template"
 R"w2c_template(
+typedef enum { RefFunc, RefNull, GlobalGet } wasm_elem_segment_expr_type_t;
+)w2c_template"
+R"w2c_template(
 typedef struct {
 )w2c_template"
-R"w2c_template(  enum { RefFunc, RefNull, GlobalGet } expr_type;
+R"w2c_template(  wasm_elem_segment_expr_type_t expr_type;
 )w2c_template"
 R"w2c_template(  wasm_rt_func_type_t type;
 )w2c_template"
